/************************************************************************************************/
/** Drop all _cvp objects                                                                      **/
/************************************************************************************************/
DECLARE @lorsData CURSOR, @lcNAME NVARCHAR(128), @lcTYPE NVARCHAR(128), @lcSQL NVARCHAR(4000)
SET @lorsData=CURSOR LOCAL READ_ONLY FOR
    SELECT AB.Name+'.'+AA.NAME, AA.TYPE
      FROM SYS.ALL_OBJECTS AA INNER JOIN sys.schemas AB ON AA.schema_ID=AB.schema_ID
      WHERE AA.NAME LIKE '[_]CVP[_]%'

OPEN @lorsData
WHILE 0 = 0 BEGIN
  FETCH NEXT FROM @lorsData INTO @lcNAME, @lcTYPE
  IF @@FETCH_STATUS <> 0 BREAK
 
  SET @lcSQL='DROP '+CASE WHEN @lcTYPE='P' THEN 'PROCEDURE' ELSE 'FUNCTION' END+' '+@lcNAME
  EXEC (@lcSQL)
END
CLOSE @lorsData
DEALLOCATE @lorsData
GO

IF NOT OBJECT_ID('dbo._CVP_CHECKGRANT') IS NULL DROP PROCEDURE dbo._CVP_CHECKGRANT
GO
CREATE PROCEDURE dbo._CVP_CHECKGRANT
AS
BEGIN
 DECLARE @lorsData CURSOR
 DECLARE @lcNAME NVARCHAR(128), @liEXISTS INT, @lcLIST NVARCHAR(MAX)

 PRINT 'bulk admin role: '+ CASE WHEN IS_SRVROLEMEMBER('bulkadmin')=1 THEN 'Yes' ELSE 'No' END

 IF ISNULL(HAS_DBACCESS('master'),0)=0 BEGIN
    SET @lcLIST='/*****************************************/'+CHAR(10)+
                '/** RUN this                            **/'+CHAR(10)+
                '/*****************************************/'+CHAR(10)+
                REPLACE('USE master; CREATE USER [%LOGIN%] FOR LOGIN [%LOGIN%];'+CHAR(10)+'GO'+CHAR(10),'%LOGIN%',SYSTEM_USER)
    PRINT @lcLIST
    END
 ELSE BEGIN
    PRINT REPLACE('USER %LOGIN% has access to database master', '%LOGIN%',SYSTEM_USER)
 END

 IF (SELECT COUNT(1) FROM fn_my_permissions(NULL, 'server') WHERE permission_name='ALTER TRACE')=0 BEGIN
    SET @lcLIST='/*****************************************/'+CHAR(10)+
                '/** RUN this                            **/'+CHAR(10)+
                '/*****************************************/'+CHAR(10)+
                REPLACE('GRANT ALTER TRACE TO [%LOGIN%];'+CHAR(10)+'GO'+CHAR(10),'%LOGIN%',SYSTEM_USER)
    PRINT @lcLIST
    END
 ELSE BEGIN
    PRINT REPLACE('USER %LOGIN% has "ALTER TRACE permissions"', '%LOGIN%',SYSTEM_USER)
 END

 IF (SELECT SUM(CASE WHEN VALUE=1 THEN 1 ELSE 0 END) FROM sys.configurations WHERe NAME IN ('show advanced options','Ole Automation Procedures'))<2 BEGIN
    SET @lcLIST='/*****************************************/'+CHAR(10)+
                '/** RUN this                            **/'+CHAR(10)+
                '/*****************************************/'+CHAR(10)+
                'sp_configure ''show advanced options'', 1;'+CHAR(10)+
                'GO'+CHAR(10)+
                'RECONFIGURE; '+CHAR(10)+
                'GO'+CHAR(10)+
                'sp_configure ''Ole Automation Procedures'', 1;'+CHAR(10)+
                'GO'+CHAR(10)+
                'RECONFIGURE; '+CHAR(10)+
                'GO'+CHAR(10)

    PRINT @lcLIST
    END
 ELSE BEGIN
    PRINT 'Server has "Ole Automation Procedures" set to 1'
 END

 SET @lcLIST=''
 SET @lorsData=CURSOR LOCAL READ_ONLY FOR
  SELECT BA.name, ISNULL(BB.ISEXISTS,0)
    FROM (SELECT 'sp_OACreate' AS NAME
          UNION 
          SELECT 'sp_OADestroy'
          UNION 
          SELECT 'sp_OAGetErrorInfo'
          UNION 
          SELECT 'sp_OAMethod') BA LEFT JOIN (SELECT AB.name, CASE WHEN AA.permission_name='EXECUTE' THEN 1 ELSE 0 END AS ISEXISTS
                                                FROM master.sys.database_permissions AA JOIN sys.system_objects AB ON AA.major_id = AB.object_id
                                                WHERE AA.class = 1 AND AB.parent_object_id = 0 AND AB.name IN ('sp_OACreate','sp_OADestroy','sp_OAGetErrorInfo','sp_OAMethod')
                                              ) BB ON BA.NAME=BB.NAME
 OPEN @lorsData
 WHILE 0 = 0 BEGIN
   FETCH NEXT FROM @lorsData INTO @lcNAME, @liEXISTS
   IF @@FETCH_STATUS <> 0 BREAK

   PRINT @lcNAME+' '+CASE WHEN @liEXISTS=1 THEN 'GRANTED' ELSE 'NONGRANTED' END
 
   IF @liEXISTS=0 BEGIN
      SET @lcLIST=@lcLIST+'GRANT EXEC ON '+@lcNAME+' TO ['+SYSTEM_USER+']'+CHAR(10)+'GO'+CHAR(10)
   END
   
 END
 CLOSE @lorsData
 DEALLOCATE @lorsData

 IF LEN(@lcLIST)>0 BEGIN
    PRINT ''
    PRINT 'Missing grant of procedure'
    PRINT '/*****************************************/'+CHAR(10)+
          '/** RUN this                            **/'+CHAR(10)+
          '/*****************************************/'+CHAR(10)+
          'use master'+CHAR(10)+
		  @lcLIST
    PRINT ''
 END



END --_CVP_CHECKGRANT
GO

--exec dbo._CVP_CHECKGRANT


IF NOT OBJECT_ID('dbo._CVP_SAVESRCS') IS NULL DROP PROCEDURE dbo._CVP_SAVESRCS
GO
CREATE PROCEDURE dbo._CVP_SAVESRCS(@lcFOLDER NVARCHAR(1000), @lcSHORTFILENAME NVARCHAR(1000), @liMODE INT=1)
AS
BEGIN
 DECLARE @lorsData CURSOR, @lorsDB CURSOR
 DECLARE @liErr INT, @liErrX INT, @loFSO INT, @loTXT INT, @lcMSG VARCHAR(MAX), @liFLAG INT, @lcFile VARCHAR(2000),
         @lcLINE VARCHAR(4000), @liS BIGINT, @liDATABASEID INT, @lcNAME NVARCHAR(256), @liID INT, 
         @loLOGIF INT, @loLOGIP INT, @loLOGIS INT, @lcLOGIF VARCHAR(2000), @lcLOGIP VARCHAR(2000), @lcLOGIS VARCHAR(2000),
         @liDBID INT, @liPID INT, @liOBJECTID INT

 DECLARE @lcFILENAME VARCHAR(100), @lcTEXT NVARCHAR(MAX), @liTLEN BIGINT

BEGIN TRY

 EXECUTE @liErr = sp_OACreate  'Scripting.FileSystemObject' , @loFSO OUT

 IF @liErr<>0 GOTO _CVP_SAVESRCS_END

 SET @lcFOLDER=@lcFOLDER+CASE WHEN RIGHT(@lcFOLDER,1)<>'\' THEN '\' ELSE '' END


 -- kontrola zda existuje sloka
 EXECUTE @liErr = sp_OAMethod   @loFSO, 'FolderExists',  @liFLAG OUT, @lcFOLDER
 IF @liErr<>0 BEGIN 
    GOTO _CVP_SAVESRCS_END
 END 
 IF @liFLAG=0 BEGIN
    SET @lcMSG=REPLACE('Neexistuje adres pro uloen soubor %DIR%. Export zruen.','%DIR%',@lcFOLDER)
    PRINT @lcMSG
    GOTO _CVP_SAVESRCS_END
 END

 -- vytvoen statistickho souboru
 SET @lcLOGIS=@lcFOLDER+@lcSHORTFILENAME+'.logis'
 EXECUTE @liErr = sp_OAMethod   @loFSO, 'CreateTextFile',  @loLOGIS OUT, @lcLOGIS, 2, 0
 IF @liErr<>0 BEGIN 
    SET @lcMSG=REPLACE('Nelze vytvoit soubor %FILE%.','%FILE%',@lcLOGIS)
    PRINT @lcMSG
    GOTO _CVP_SAVESRCS_TXT
 END 
 SET @lcLINE=(SELECT CAST(DATABASEIDLEN AS VARCHAR(30))+','+CAST(OBJECTIDLEN AS VARCHAR(30))+','+CAST(NESTLEVELLEN AS VARCHAR(30)) FROM #STATS)
 EXECUTE @liErr = sp_OAMethod  @loLOGIS, 'WriteLine', Null, @lcLINE
 IF @liErr=0 BEGIN
    SET @lcMSG=REPLACE('Export dat do souboru %FILE%.','%FILE%',@lcLOGIS)
    PRINT @lcMSG
 END
 EXECUTE @liErrX = sp_OAMethod  @loLOGIS, 'Close'
 IF @liErr=0 BEGIN
    EXECUTE @liErr = sp_OAMethod   @loFSO, 'DELETEFILE',  NULL, @lcFOLDER, 1
 END
 EXECUTE sp_OADestroy @loTXT

 _CVP_SAVESRCS_TXT:

 IF @liMODE=2 BEGIN
    SET @lcLOGIF=@lcFOLDER+@lcSHORTFILENAME+'.logif'
    SET @lcLOGIP=@lcFOLDER+@lcSHORTFILENAME+'.logip'

    -- vytvoen textovho souboru
    EXECUTE @liErr = sp_OAMethod   @loFSO, 'CreateTextFile',  @loLOGIF OUT, @lcLOGIF, 2, 0
    IF @liErr<>0 BEGIN 
       SET @lcMSG=REPLACE('Nelze vytvoit soubor %FILE%. Export zruen.','%FILE%',@lcLOGIF)
       PRINT @lcMSG
       GOTO _CVP_SAVESRCS_END
    END 

    -- vytvoen textovho souboru
    EXECUTE @liErr = sp_OAMethod   @loFSO, 'CreateTextFile',  @loLOGIP OUT, @lcLOGIP, 2, 0
    IF @liErr<>0 BEGIN 
       SET @lcMSG=REPLACE('Nelze vytvoit soubor %FILE%. Export zruen.','%FILE%',@lcLOGIP)
       PRINT @lcMSG
       GOTO _CVP_SAVESRCS_END
    END 
 END

 SET @lorsDB=CURSOR LOCAL READ_ONLY FOR
   SELECT ID, DATABASEID, FILENAME
     FROM #DBs
     WHERE DATABASEID IN (SELECT DATABASEID FROM #OBJECTs)
 

 OPEN @lorsDB
 WHILE 0 = 0 BEGIN
   FETCH NEXT FROM @lorsDB INTO @liDBID, @liDATABASEID, @lcFILENAME
   IF @@FETCH_STATUS <> 0 BREAK

   SET @lcFile=@lcFOLDER+@lcFILENAME

   IF @liMODE=2 BEGIN
      SET @lcLINE=CAST(@liDBID AS VARCHAR(30))+','+@lcFILENAME+','+CAST(@liDATABASEID AS VARCHAR(30))
      EXECUTE @liErr = sp_OAMethod  @loLOGIF, 'WriteLine', Null, @lcLINE
      IF @liErr<>0 BEGIN PRINT 'BEM''s ATTACK' BREAK END
   END

   -- vytvoen textovho souboru
   EXECUTE @liErr = sp_OAMethod   @loFSO, 'CreateTextFile',  @loTXT OUT, @lcFile, 2, 0
   IF @liErr<>0 BEGIN 
      SET @lcMSG=REPLACE('Nelze vytvoit soubor %FILE%. Export zruen.','%FILE%',@lcFile)
      PRINT @lcMSG
      BREAK
   END 

   SET @lorsData=CURSOR LOCAL READ_ONLY FOR
     SELECT ID, CAST(SOURCE AS NVARCHAR(MAX)), LEN('a'+CAST(SOURCE AS NVARCHAR(MAX))+'a')-2 AS TLEN, OWNER+'.'+NAME, OBJECTID
       FROM #OBJECTs 
       WHERE DATABASEID=@liDATABASEID
       ORDER BY ID

  OPEN @lorsData
   WHILE 0 = 0 BEGIN
     FETCH NEXT FROM @lorsData INTO @liPID, @lcTEXT, @liTLEN, @lcNAME, @liOBJECTID
     IF @@FETCH_STATUS <> 0 BREAK

     IF @liMODE=2 BEGIN
        SET @lcLINE=CAST(@liDBID AS VARCHAR(30))+','+CAST(@liPID AS VARCHAR(30))+',,'+@lcNAME+','+CAST(@liOBJECTID AS VARCHAR(30))
        EXECUTE @liErr = sp_OAMethod  @loLOGIP, 'WriteLine', Null, @lcLINE
        IF @liErr<>0 BEGIN PRINT 'BEM''s ATTACK' BREAK END
     END

     IF @liTLEN<=4000 AND RIGHT(@lcTEXT,1)<>CHAR(10) BEGIN
        EXECUTE @liErr = sp_OAMethod  @loTXT, 'WriteLine', Null, @lcTEXT
        IF @liErr<>0 BEGIN PRINT 'BEM''s ATTACK' BREAK END
     END
     IF @liTLEN<=4000 AND RIGHT(@lcTEXT,1)=CHAR(10) BEGIN
        EXECUTE @liErr = sp_OAMethod  @loTXT, 'Write', Null, @lcTEXT
        IF @liErr<>0 BEGIN PRINT 'BEM''s ATTACK' BREAK END
     END
     IF @liTLEN>4000  BEGIN
        SET @liS=0
        WHILE @liS<=@liTLEN BEGIN
          SET @lcLINE=SUBSTRING(@lcTEXT,@liS+1,4000)

          EXECUTE @liErr = sp_OAMethod  @loTXT, 'Write', Null, @lcLINE
          IF @liErr<>0 BEGIN PRINT 'BEM''s ATTACK' BREAK END

          SET @liS=@liS+4000
        END 
     END
     --END OF @loIFT061_TLEN>4000
     EXECUTE @liErr = sp_OAMethod  @loTXT, 'WriteLine', Null, ''
     IF @liErr<>0 BEGIN PRINT 'BEM''s ATTACK' BREAK END

   END
   CLOSE @lorsData
   DEALLOCATE @lorsData



   -- zavi vstupn soubor
   EXECUTE @liErrX = sp_OAMethod  @loTXT, 'Close'
   SET @liErr=@liErr|@liErrX
   EXECUTE sp_OADestroy @loTXT

   IF @liErr=0 BEGIN
      SET @lcMSG=REPLACE('Export dat do souboru %FILE%.','%FILE%',@lcFile)
      PRINT @lcMSG
   END

   IF @liErr<>0 BEGIN
      SET @lcMSG=REPLACE('Nelze vytvoit soubor %FILE%. Export zruen.','%FILE%',@lcFile)
      PRINT @lcMSG
   END


 END
 CLOSE @lorsDB
 DEALLOCATE @lorsDB


 IF @liMODE=2 BEGIN
    EXECUTE @liErr = sp_OAMethod  @loLOGIF, 'Close'
    EXECUTE sp_OADestroy @loLOGIF

    IF @liErr=0 BEGIN
       SET @lcMSG=REPLACE('Export dat do souboru %FILE%.','%FILE%',@lcLOGIF)
       PRINT @lcMSG
    END
    IF @liErr<>0 BEGIN
       SET @lcMSG=REPLACE('Nelze vytvoit soubor %FILE%. Export zruen.','%FILE%',@lcLOGIF)
       PRINT @lcMSG
    END

    EXECUTE @liErr = sp_OAMethod  @loLOGIP, 'Close'
    EXECUTE sp_OADestroy @loLOGIP

    IF @liErr=0 BEGIN
       SET @lcMSG=REPLACE('Export dat do souboru %FILE%.','%FILE%',@lcLOGIP)
       PRINT @lcMSG
    END
    IF @liErr<>0 BEGIN
       SET @lcMSG=REPLACE('Nelze vytvoit soubor %FILE%. Export zruen.','%FILE%',@lcLOGIP)
       PRINT @lcMSG
    END

 END




 _CVP_SAVESRCS_END:
 EXECUTE sp_OADestroy @loFSO

END TRY

BEGIN CATCH
 SET @lcMSG=CAST(ERROR_PROCEDURE() AS VARCHAR(30))+' '+CAST(ERROR_NUMBER() AS VARCHAR(30))+' '+ERROR_MESSAGE()
 PRINT '***********************************************************'
 PRINT @lcMSG
 PRINT '***********************************************************'
END CATCH

END --_CVP_SAVESRCS
GO

IF NOT OBJECT_ID('dbo._CVP_SAVELOG') IS NULL DROP PROCEDURE dbo._CVP_SAVELOG
GO
CREATE PROCEDURE dbo._CVP_SAVELOG(@lcFOLDER NVARCHAR(1000), @lcSHORTFILENAME NVARCHAR(1000), @lcTABLENAME NVARCHAR(128), @liMODE INT=1, @liMAXNESTLEVEL INT=0 OUT)
AS
BEGIN
 DECLARE @lorsData CURSOR
 DECLARE @liErr INT, @liErrX INT, @loFSO INT, @loTXT INT, @lcMSG VARCHAR(MAX), @liFLAG INT, @lcFile VARCHAR(2000),
         @lcLINE VARCHAR(4000), @liS BIGINT, @lcBUFFER VARCHAR(4000), @liNESTLEVELZ INT, @ltT1 DATETIME

 DECLARE @liTLEN BIGINT, @lcSQL NVARCHAR(MAX), @lcDQ NVARCHAR(MAX)
 DECLARE @ldSTARTTIME DATETIME, @lcPROC NVARCHAR(256), @liLINENUMBER INT, @lcFILENAME NVARCHAR(1000), 
         @liLVL INT, @lnDURATION NUMERIC(20,6), @liROWNUMBER INT, @liREQUESTID INT, @liDQ INT, @liOBJECTTYPE INT, @liEVENTCLASS INT, @liEVENTSEQUENCE BIGINT


 EXECUTE @liErr = sp_OACreate  'Scripting.FileSystemObject' , @loFSO OUT

 IF @liErr<>0 GOTO _CVP_SAVELOG_END

 SET @lcFOLDER=@lcFOLDER+CASE WHEN RIGHT(@lcFOLDER,1)<>'\' THEN '\' ELSE '' END

 SET @lcFile=@lcFOLDER+@lcSHORTFILENAME+'.log'

 -- kontrola zda existuje sloka
 EXECUTE @liErr = sp_OAMethod   @loFSO, 'FolderExists',  @liFLAG OUT, @lcFOLDER
 IF @liErr<>0 BEGIN 
    GOTO _CVP_SAVELOG_END
 END 
 IF @liFLAG=0 BEGIN
    SET @lcMSG=REPLACE('Neexistuje adres pro uloen soubor %DIR%. Export zruen.','%DIR%',@lcFOLDER)
    PRINT @lcMSG
    GOTO _CVP_SAVELOG_END
 END

 -- vytvoen textovho souboru
 EXECUTE @liErr = sp_OAMethod   @loFSO, 'CreateTextFile',  @loTXT OUT, @lcFile, 2, 0
 IF @liErr<>0 BEGIN 
    SET @lcMSG=REPLACE('Nelze vytvoit soubor %FILE%. Export zruen.','%FILE%',@lcFile)
    PRINT @lcMSG
    GOTO _CVP_SAVELOG_END
 END 

 --  SET @ltT1=GETDATE()
 --   SET @lcSQL='
 --     SELECT AG.DURATION/1000000.0000, AB.OWNER+''.''+AB.NAME, AE.LINENUMBER, AF.FILENAME, 
 --            AA.NESTLEVEL+CASE WHEN AB.DQ=1 THEN CASE WHEN (SELECT TOP 1 BC.EVENTCLASS FROM '+@lcTABLENAME+' BC WHERE BC.EVENTSEQUENCE<AA.EVENTSEQUENCE ORDER BY BC.EVENTSEQUENCE DESC)=11 THEN -1 ELSE 0 END ELSE 0 END, AA.REQUESTID
 --       FROM '+@lcTABLENAME+' AA INNER JOIN #OBJECTs  AB ON AA.DATABASEID=AB.DATABASEID AND AA.OBJECTID=AB.OBJECTID AND CASE WHEN AB.DQ=1 THEN AA.EVENTSEQUENCE ELSE -1 END=AB.EVENTSEQUENCE
 --                          INNER JOIN #OBJECTSRCs AE ON AA.DATABASEID=AE.DATABASEID AND AB.OBJECTID=AE.OBJECTID AND CASE WHEN AB.DQ=1 THEN AA.EVENTSEQUENCE ELSE -1 END=AE.EVENTSEQUENCE AND AA.LINENUMBER=AE.ORD
 --                          INNER JOIN #DBs        AF ON AA.DATABASEID=AF.DATABASEID
 --                          INNER JOIN '+@lcTABLENAME+' AG ON AG.EVENTSEQUENCE=(SELECT MIN(BB.EVENTSEQUENCE)
 --                                                                                FROM '+@lcTABLENAME+' BB
 --                                                                                WHERE AA.OFFSET=BB.OFFSET AND BB.EVENTSEQUENCE>AA.EVENTSEQUENCE AND EVENTCLASS=45
 --                                                                              ) AND AG.EVENTCLASS=45
 --       WHERE AA.EVENTCLASS=44
 --       ORDER BY AA.EVENTSEQUENCE
 --       '
 --EXEC (@lcSQL)
 --PRINT 'LOGDATA '+CAST(DATEDIFF(ms,@ltT1,GETDATE())/1000.0 AS VARCHAR(30))


 --SET @lcSQL='
 --     SELECT AA.EVENTSEQUENCE, AA.OBJECTTYPE, AG.DURATION/1000000.0000, AA.OBJECTID, AB.ID AS OBJECTIDX, AE.LINENUMBER, AF.ID AS FILEID, AA.NESTLEVEL, AA.REQUESTID
 --       FROM '+@lcTABLENAME+' AA LEFT JOIN #OBJECTs  AB ON AA.DATABASEID=AB.DATABASEID AND AA.OBJECTID=AB.OBJECTID AND CASE WHEN AB.DQ=1 THEN AA.EVENTSEQUENCE ELSE -1 END=AB.EVENTSEQUENCE
 --                          LEFT JOIN #OBJECTSRCs AE ON AB.ID=AE.ID AND AA.LINENUMBER=AE.ORD
 --                          LEFT JOIN #DBs        AF ON AA.DATABASEID=AF.DATABASEID
 --                          LEFT JOIN '+@lcTABLENAME+' AG ON AG.EVENTSEQUENCE=(SELECT MIN(BB.EVENTSEQUENCE)
 --                                                                                FROM '+@lcTABLENAME+' BB
 --                                                                                WHERE AA.OFFSET=BB.OFFSET AND BB.EVENTSEQUENCE>AA.EVENTSEQUENCE AND EVENTCLASS=45
 --                                                                              ) AND AG.EVENTCLASS=45
 --       WHERE AA.EVENTCLASS=44
 --       ORDER BY AA.EVENTSEQUENCE
 --    '
 --EXEC (@lcSQL)

 SET @liDQ=(SELECT COUNT(1) FROM #OBJECTs WHERE DQ=1)
 SET @lcDQ=CASE WHEN @liDQ=0 THEN '' ELSE 'SELECT TOP 1 @liEVENTCLASS=EVENTCLASS FROM '+@lcTABLENAME+' BC WHERE EVENTSEQUENCE<@liEVENTSEQUENCE ORDER BY EVENTSEQUENCE DESC' END

 -- Vytvo dle zdrojov tabulky log soubor pro VFP
 SET @ltT1=GETDATE()
 IF @liMODE=1 BEGIN
    SET @lcSQL='SET @loCursor = CURSOR LOCAL READ_ONLY FOR 
      SELECT AA.EVENTSEQUENCE, AA.OBJECTTYPE, AG.DURATION/1000000.0000, AB.OWNER+''.''+AB.NAME, AE.LINENUMBER, AF.FILENAME, AA.NESTLEVEL, AA.REQUESTID
        FROM '+@lcTABLENAME+' AA INNER JOIN #OBJECTs  AB ON AA.DATABASEID=AB.DATABASEID AND AA.OBJECTID=AB.OBJECTID AND CASE WHEN AB.DQ=1 THEN AA.EVENTSEQUENCE ELSE -1 END=AB.EVENTSEQUENCE
                           INNER JOIN #OBJECTSRCs AE ON AB.ID=AE.ID AND AA.LINENUMBER=AE.ORD
                           INNER JOIN #DBs        AF ON AA.DATABASEID=AF.DATABASEID
                           INNER JOIN '+@lcTABLENAME+' AG ON AG.EVENTSEQUENCE=(SELECT MIN(BB.EVENTSEQUENCE)
                                                                                 FROM '+@lcTABLENAME+' BB
                                                                                 WHERE AA.OFFSET=BB.OFFSET AND BB.EVENTSEQUENCE>AA.EVENTSEQUENCE AND EVENTCLASS=45
                                                                               ) AND AG.EVENTCLASS=45
        WHERE AA.EVENTCLASS=44
        ORDER BY AA.EVENTSEQUENCE
        ;
       OPEN @loCursor'
 END

 IF @liMODE=2 BEGIN
    SET @lcSQL='SET @loCursor = CURSOR LOCAL READ_ONLY FOR 
      SELECT AA.EVENTSEQUENCE, AA.OBJECTTYPE, AG.DURATION/1000000.0000, AB.ID, AE.LINENUMBER, AF.ID, AA.NESTLEVEL, AA.REQUESTID
        FROM '+@lcTABLENAME+' AA INNER JOIN #OBJECTs  AB ON AA.DATABASEID=AB.DATABASEID AND AA.OBJECTID=AB.OBJECTID AND CASE WHEN AB.DQ=1 THEN AA.EVENTSEQUENCE ELSE -1 END=AB.EVENTSEQUENCE
                           INNER JOIN #OBJECTSRCs AE ON AB.ID=AE.ID AND AA.LINENUMBER=AE.ORD
                           INNER JOIN #DBs        AF ON AA.DATABASEID=AF.DATABASEID
                           INNER JOIN '+@lcTABLENAME+' AG ON AG.EVENTSEQUENCE=(SELECT MIN(BB.EVENTSEQUENCE)
                                                                                 FROM '+@lcTABLENAME+' BB
                                                                                 WHERE AA.OFFSET=BB.OFFSET AND BB.EVENTSEQUENCE>AA.EVENTSEQUENCE AND EVENTCLASS=45
                                                                               ) AND AG.EVENTCLASS=45
        WHERE AA.EVENTCLASS=44
        ORDER BY AA.EVENTSEQUENCE
        ;
       OPEN @loCursor'
 END

 EXEC sp_executesql @lcSQL, N'@loCursor CURSOR OUTPUT', @lorsData OUTPUT
 PRINT 'OPEN CURSOR '+CAST(DATEDIFF(ms,@ltT1,GETDATE())/1000.0 AS VARCHAR(30))

 SET @liROWNUMBER=0
 SET @lcBUFFER=''
 SET @liNESTLEVELZ=0
 WHILE 0 = 0 BEGIN
   FETCH NEXT FROM @lorsData INTO @liEVENTSEQUENCE, @liOBJECTTYPE, @lnDURATION, @lcPROC, @liLINENUMBER, @lcFILENAME, @liLVL, @liREQUESTID
   IF @@FETCH_STATUS <> 0 BREAK

   IF @liOBJECTTYPE IN (20801,20816) BEGIN
      EXEC sp_executesql @lcDQ, N'@liEVENTCLASS INT OUTPUT, @liEVENTSEQUENCE BIGINT', @liEVENTCLASS OUTPUT, @liEVENTSEQUENCE
      SET @liLVL=@liLVL+CASE WHEN @liEVENTCLASS=11 THEN -1 ELSE 0 END
   END

   SELECT @liMAXNESTLEVEL= CASE WHEN @liMAXNESTLEVEL<LVL THEN LVL ELSE @liMAXNESTLEVEL END,
          @liNESTLEVELZ=NESTLEVELZ, @liLVL=LVL
     FROM (SELECT CASE WHEN @liREQUESTID=0 THEN @liLVL ELSE @liNESTLEVELZ END AS NESTLEVELZ,
                  CASE WHEN @liREQUESTID<>0 THEN @liLVL+@liNESTLEVELZ ELSE @liLVL END AS LVL
          ) AA

   SET @liROWNUMBER=@liROWNUMBER+1
   SET @lcLINE=CONVERT(VARCHAR(20),@lnDURATION,0)+',,'+@lcPROC+','+CONVERT(VARCHAR(30),@liLINENUMBER,0)+','+@lcFILENAME+','+CONVERT(VARCHAR(30),@liLVL,0)+CHAR(13)+CHAR(10)

   IF LEN(@lcBUFFER)+LEN(@lcLINE)>4000 BEGIN
      EXECUTE @liErr = sp_OAMethod  @loTXT, 'Write', Null, @lcBUFFER
      IF @liErr<>0 BEGIN PRINT 'BEM''s ATTACK' BREAK END
      SET @lcBUFFER=''
      PRINT @liROWNUMBER
   END
   SET @lcBUFFER=@lcBUFFER+@lcLINE

 END
 CLOSE @lorsData
 DEALLOCATE @lorsData

 IF LEN(@lcBUFFER)>0 BEGIN
    EXECUTE @liErr = sp_OAMethod  @loTXT, 'Write', Null, @lcBUFFER
    IF @liErr<>0 BEGIN PRINT 'BEM''s ATTACK' END
 END

 -- zavi vstupn soubor
 EXECUTE @liErrX = sp_OAMethod  @loTXT, 'Close'
 SET @liErr=@liErr|@liErrX
 EXECUTE sp_OADestroy @loTXT

 IF @liErr=0 BEGIN
    SET @lcMSG=REPLACE('Export dat do souboru %FILE%.','%FILE%',@lcFile)
    PRINT @lcMSG
 END

 IF @liErr<>0 BEGIN
    SET @lcMSG=REPLACE('Nelze vytvoit soubor %FILE%. Export zruen.','%FILE%',@lcFile)
    PRINT @lcMSG
 END


 _CVP_SAVELOG_END:
 EXECUTE sp_OADestroy @loFSO


END --_CVP_SAVELOG
GO

IF NOT OBJECT_ID('dbo._CVP_SS') IS NULL DROP FUNCTION dbo._CVP_SS
GO


IF NOT OBJECT_ID('dbo._CVP_PROCESS') IS NULL DROP PROCEDURE dbo._CVP_PROCESS
GO
CREATE PROCEDURE dbo._CVP_PROCESS(@lcTABLENAME NVARCHAR(128), @lcFOLDER NVARCHAR(1000), @lcSHORTFILENAME NVARCHAR(1000), @liMODE INT=1)
AS
BEGIN

 DECLARE @lorsDB CURSOR, @lorsOBJECT CURSOR
 DECLARE @lcSQL NVARCHAR(MAX), @lcSOURCE NVARCHAR(MAX),@lcDQ NVARCHAR(MAX)
 DECLARE @liDATABASEID INT, @lcNAME NVARCHAR(128), @liOBJECTID INT, @liCNT INT, @liROWS INT, @liMAXNESTLEVEL INT, @liDATABASEIDX INT,
         @liEVENTSEQUENCE INT, @liID INT
 DECLARE @ltT1 DATETIME

 CREATE TABLE #DBs (ID INT, DATABASEID INT, NAME NVARCHAR(128), FILENAME NVARCHAR(100))
 CREATE TABLE #OBJECTs (ID INT, DATABASEID INT, OBJECTID INT, EVENTSEQUENCE INT, OWNER NVARCHAR(128), NAME NVARCHAR(128), SOURCE NVARCHAR(MAX), DQ NUMERIC(1), UNIQUE CLUSTERED (DATABASEID, OBJECTID, EVENTSEQUENCE))
 CREATE TABLE #OBJECTSRCs (ID INT, LINENUMBER INT, ORD INT, UNIQUE CLUSTERED (ID, ORD))
 CREATE TABLE #STATS (DATABASEIDLEN INT, OBJECTIDLEN INT, NESTLEVELLEN INT)

 -- zjitn databz
 SET @ltT1=GETDATE()
 SET @lcSQL=
 'INSERT INTO #DBs '+
 '  SELECT ROW_NUMBER() OVER( ORDER BY AB.NAME), AA.DATABASEID, AB.NAME, REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE('''+@lcSHORTFILENAME+'_''+CAST(AA.DATABASEID AS VARCHAR(30)),''['',''''), ''\'',''''), ''/'',''''), '':'',''''), ''*'',''''), ''?'',''''), ''"'',''''), ''<'',''''), ''>'',''''), ''|'',''''), '']'','''')+''.tsql'''+
 '    FROM (SELECT DATABASEID FROM '+@lcTABLENAME+' WHERE EVENTCLASS=44 GROUP BY DATABASEID'+
 '         ) AA INNER JOIN sys.databases  AB ON AA.databaseid=AB.database_id'
 EXECUTE (@lcSQL)
 PRINT 'DBLists '+CAST(DATEDIFF(ms,@ltT1,GETDATE())/1000.0 AS VARCHAR(30))


 SET @lcDQ='''CREATE PROCEDURE dsql.[''+CAST(EVENTSEQUENCE AS VARCHAR(30))+''] AS ''+CHAR(13)+CHAR(10)+''BEGIN''+CHAR(13)+CHAR(10)+ISNULL(CAST(TEXTDATA AS NVARCHAR(MAX)),''EXECUTE SQL'')+CHAR(13)+CHAR(10)+''END''+CHAR(13)+CHAR(10)'

 -- zjitn procedur a funkc v databzch
 SET @lorsDB=CURSOR READ_ONLY STATIC FOR
   SELECT DATABASEID, NAME FROM #DBs
 OPEN @lorsDB
 SET @liCNT=0
 WHILE 0 = 0 BEGIN
   FETCH NEXT FROM @lorsDB INTO @liDATABASEID, @lcNAME
   IF @@FETCH_STATUS <> 0 BREAK

   SET @ltT1=GETDATE()
   SET @lcSQL=
   'INSERT INTO #OBJECTs '+
   '  SELECT ROW_NUMBER() OVER( ORDER BY AC.NAME, AB.NAME)+'+CAST(@liCNT AS VARCHAR(30))+', '+
   '         AA.DATABASEID, AA.OBJECTID, -1, AC.NAME, AB.NAME, AD.[definition], 0'+
   '    FROM (SELECT DATABASEID, OBJECTID FROM '+@lcTABLENAME+' WHERE DATABASEID='+CAST(@liDATABASEID AS VARCHAR(30))+' AND EVENTCLASS=44 AND NOT ObjectType IN (20801,20816) GROUP BY DATABASEID, OBJECTID'+
   '         ) AA INNER JOIN '+@lcNAME+'.sys.all_objects     AB ON AA.objectid=ABS(AB.object_id)'+
   '              INNER JOIN '+@lcNAME+'.sys.schemas         AC ON AB.schema_id=AC.schema_id'+
   '              INNER JOIN '+@lcNAME+'.sys.all_sql_modules AD ON AB.object_id=AD.object_id'+
   '    ORDER BY AC.NAME, AB.NAME'
   EXECUTE (@lcSQL)
   SET @liCNT=@liCNT+@@ROWCOUNT
   PRINT 'OBJECTs '+CAST(DATEDIFF(ms,@ltT1,GETDATE())/1000.0 AS VARCHAR(30))


   SET @ltT1=GETDATE()
   SET @lcSQL=
   'INSERT INTO #OBJECTs '+
   '  SELECT ROW_NUMBER() OVER( ORDER BY EVENTSEQUENCE)+'+CAST(@liCNT AS VARCHAR(30))+', '+
   '         DATABASEID, OBJECTID, EVENTSEQUENCE, ''dsql'', CAST(EVENTSEQUENCE AS VARCHAR(30)), '+
   '         '+@lcDQ+',1'+
   '    FROM '+@lcTABLENAME+
   '    WHERE DATABASEID='+CAST(@liDATABASEID AS VARCHAR(30))+' AND EVENTCLASS=44 AND ObjectType IN (20801,20816)'
   EXECUTE (@lcSQL)
   SET @liCNT=@liCNT+@@ROWCOUNT
   PRINT 'DSQLs '+CAST(DATEDIFF(ms,@ltT1,GETDATE())/1000.0 AS VARCHAR(30))

 END 
 CLOSE @lorsDB
 DEALLOCATE @lorsDB

 SET @ltT1=GETDATE()
 SET @liDATABASEIDX=0
 -- zjitn zdrojovch kd procedur a funkc v databzch
 SET @lorsOBJECT=CURSOR READ_ONLY STATIC FOR
   SELECT DATABASEID, LEN(SOURCE)-LEN(REPLACE(SOURCE,CHAR(10),''))+1, ID
     FROM #OBJECTs
     WHERE NOT SOURCE IS NULL
     ORDER BY DATABASEID, ID
 OPEN @lorsOBJECT
 WHILE 0 = 0 BEGIN
   FETCH NEXT FROM @lorsOBJECT INTO @liDATABASEID, @liROWS, @liID
   IF @@FETCH_STATUS <> 0 BREAK;

   IF @liDATABASEIDX<>@liDATABASEID BEGIN
      SET @liCNT=0
   END ;

   WITH tree(id) AS
    (SELECT 1
     UNION ALL
     SELECT AA.ID+AB.ID 
       FROM (SELECT 1 AS ID) AA INNER JOIN TREE AB ON 0=0
       WHERE AA.ID+AB.ID<=@LIROWS
    )
   INSERT INTO #OBJECTSRCs
     SELECT @liID, @liCNT+ID, ID FROM tree
     OPTION (MAXRECURSION 32000)

   SET @liCNT=@liCNT+@@ROWCOUNT

   SET @liDATABASEIDX=@liDATABASEID
 END 
 CLOSE @lorsOBJECT
 DEALLOCATE @lorsOBJECT
 PRINT '#OBJECTSRCs '+CAST(DATEDIFF(ms,@ltT1,GETDATE())/1000.0 AS VARCHAR(30))

 SET @ltT1=GETDATE()
 ALTER TABLE #OBJECTs ALTER COLUMN SOURCE NTEXT
 ALTER TABLE #TRC DROP COLUMN TextData
 PRINT 'DROP TEXTDATA '+CAST(DATEDIFF(ms,@ltT1,GETDATE())/1000.0 AS VARCHAR(30))

 -- Vytvo log dle zdrojovho souboru
 SET @liMAXNESTLEVEL=0
 EXEC dbo._CVP_SAVELOG @lcFOLDER, @lcSHORTFILENAME, @lcTABLENAME, @liMODE, @liMAXNESTLEVEL OUT

 SET @ltT1=GETDATE()
 INSERT INTO #STATS (DATABASEIDLEN, OBJECTIDLEN, NESTLEVELLEN)
   SELECT MAX(LEN(CAST(AB.ID AS VARCHAR(30)))), MAX(LEN(CAST(AA.ID AS VARCHAR(30)))), MAX(LEN(CAST(@liMAXNESTLEVEL AS VARCHAR(30))))
     FROM #OBJECTs AA INNER JOIN #DBs AB ON AA.DATABASEID=AB.DATABASEID
 PRINT '#STATS '+CAST(DATEDIFF(ms,@ltT1,GETDATE())/1000.0 AS VARCHAR(30))

 -- Vytvo soubory dle natench OBJECTs a napl je zdrojovma kdama
 EXEC dbo._CVP_SAVESRCS @lcFOLDER, @lcSHORTFILENAME, @liMODE

 IF NOT OBJECT_ID('tempdb.dbo.#DBs') IS NULL DROP TABLE #DBs
 IF NOT OBJECT_ID('tempdb.dbo.#OBJECTs') IS NULL DROP TABLE #OBJECTs
 IF NOT OBJECT_ID('tempdb.dbo.#OBJECTSRCs') IS NULL DROP TABLE #OBJECTSRCs
 IF NOT OBJECT_ID('tempdb.dbo.#STATS') IS NULL DROP TABLE #STATS
END --_CVP_Process
GO



IF NOT OBJECT_ID('dbo._CVP_IMPORT_TRC') IS NULL DROP PROCEDURE dbo._CVP_IMPORT_TRC
GO
CREATE PROCEDURE dbo._CVP_IMPORT_TRC(@lcTRC NVARCHAR(1000), @lcFOLDER NVARCHAR(1000), @lcSHORTFILENAME NVARCHAR(1000), @liMODE INT=1)
AS
BEGIN
 DECLARE @ltT1 DATETIME, @lcSQL VARCHAR(MAX)

 SET @ltT1=GETDATE()
 SELECT eventclass, Databaseid, CASE WHEN ObjectType IN (20801,20816) THEN 3 ELSE Linenumber END AS Linenumber, Duration, Objectid , NESTLEVEL, REQUESTID, EVENTSEQUENCE, OFFSET, SPID, COALESCE(OBJECTTYPE,-1) AS OBJECTTYPE, CASE WHEN ObjectType IN (20801,20816) THEN TEXTDATA ELSE NULL END AS TEXTDATA
   INTO #TRC
   FROM ::fn_trace_gettable(@lcTRC, DEFAULT) 
   WHERE STATE IS NULL AND eventclass IN (44,45,11)
 PRINT 'Load '+CAST(DATEDIFF(ms,@ltT1,GETDATE())/1000.0 AS VARCHAR(30))
 

    SET @ltT1=GETDATE()
 CREATE INDEX #IN1_TRC ON #TRC (DATABASEID, OBJECTID) INCLUDE(OBJECTTYPE) WHERE EVENTCLASS=44
 PRINT '#IN1 '+CAST(DATEDIFF(ms,@ltT1,GETDATE())/1000.0 AS VARCHAR(30))

 SET @ltT1=GETDATE()
 CREATE INDEX #IN3_TRC ON #TRC (OFFSET, EVENTSEQUENCE, DURATION) WHERE EVENTCLASS=45
 PRINT '#IN3 '+CAST(DATEDIFF(ms,@ltT1,GETDATE())/1000.0 AS VARCHAR(30))

 SET @ltT1=GETDATE()
 CREATE INDEX #IN4_TRC ON #TRC (EVENTSEQUENCE)
 PRINT '#IN4 '+CAST(DATEDIFF(ms,@ltT1,GETDATE())/1000.0 AS VARCHAR(30))


 SET @ltT1=GETDATE()
 EXEC _CVP_Process '#TRC', @lcFOLDER,  @lcSHORTFILENAME, @liMODE
 PRINT '_CVP_Process '+CAST(DATEDIFF(ms,@ltT1,GETDATE())/1000.0 AS VARCHAR(30))

 
 IF NOT OBJECT_ID('tempdb.dbo.#TRC') IS NULL DROP TABLE #TRC

END --_CVP_IMPORT_TRC
GO

IF NOT OBJECT_ID('dbo._CVP_COLSLIST') IS NULL DROP FUNCTION dbo._CVP_COLSLIST
GO
CREATE FUNCTION dbo._CVP_COLSLIST()
RETURNS  @loCOLS TABLE 
(COLID INT, COLNAME NVARCHAR(128), COLDESCR NVARCHAR(4000), TYPENAME NVARCHAR(128), PRIMARY KEY (COLID)
)
AS
BEGIN
 INSERT INTO @loCOLS VALUES (1,'TextData', 'Text value dependent on the event class that is captured in the trace.','ntext');
 INSERT INTO @loCols VALUES (2,'BinaryData', 'Binary value dependent on the event class captured in the trace.','image')
 INSERT INTO @loCols VALUES (3,'DatabaseID', 'ID of the database specified by the USE database statement, or the default database if no USE database statement is issued for a given connection. The value for a database can be determined by using the DB_ID function.','int')
 INSERT INTO @loCols VALUES (4,'TransactionID', 'System-assigned ID of the transaction.','bigint')
 INSERT INTO @loCols VALUES (5,'LineNumber', 'Contains the number of the line that contains the error. For events that involve Transact-SQL statements, like SP:StmtStarting, the LineNumber contains the line number of the statement in the stored procedure or batch.','int')
 INSERT INTO @loCols VALUES (6,'NTUserName', 'Microsoft Windows user name.','nvarchar')
 INSERT INTO @loCols VALUES (7,'NTDomainName', 'Windows domain to which the user belongs.','nvarchar')
 INSERT INTO @loCols VALUES (8,'HostName', 'Name of the client computer that originated the request.','nvarchar')
 INSERT INTO @loCols VALUES (9,'ClientProcessID', 'ID assigned by the client computer to the process in which the client application is running.','int')
 INSERT INTO @loCols VALUES (10,'ApplicationName', 'Name of the client application that created the connection to an instance of SQL Server. This column is populated with the values passed by the application rather than the displayed name of the program.','nvarchar')
 INSERT INTO @loCols VALUES (11,'LoginName', 'SQL Server login name of the client.','nvarchar')
 INSERT INTO @loCols VALUES (12,'SPID', 'Server Process ID assigned by SQL Server to the process associated with the client.','int')
 INSERT INTO @loCols VALUES (13,'Duration', 'Amount of elapsed time (in microseconds) taken by the event. This data column is not populated by the Hash Warning event.','bigint')
 INSERT INTO @loCols VALUES (14,'StartTime', 'Time at which the event started, when available.','datetime')
 INSERT INTO @loCols VALUES (15,'EndTime', 'Time at which the event ended. This column is not populated for starting event classes, such as SQL:BatchStarting or SP:Starting. It is also not populated by the Hash Warning event.','datetime')
 INSERT INTO @loCols VALUES (16,'Reads', 'Number of logical disk reads performed by the server on behalf of the event. This column is not populated by the Lock:Released event.','bigint')
 INSERT INTO @loCols VALUES (17,'Writes', 'Number of physical disk writes performed by the server on behalf of the event.','bigint')
 INSERT INTO @loCols VALUES (18,'CPU', 'Amount of CPU time (in milliseconds) used by the event.','int')
 INSERT INTO @loCols VALUES (19,'Permissions', 'Represents the bitmap of permissions; used by Security Auditing.','bigint')
 INSERT INTO @loCols VALUES (20,'Severity', 'Severity level of an exception.','int')
 INSERT INTO @loCols VALUES (21,'EventSubClass', 'Type of event subclass. This data column is not populated for all event classes.','int')
 INSERT INTO @loCols VALUES (22,'ObjectID', 'System-assigned ID of the object.','int')
 INSERT INTO @loCols VALUES (23,'Success', 'Success of the permissions usage attempt; used for auditing.'+CHAR(13)+CHAR(10)+'1 = success0 = failure','int')
 INSERT INTO @loCols VALUES (24,'IndexID', 'ID for the index on the object affected by the event. To determine the index ID for an object, use the indid column of the sysindexes system table.','int')
 INSERT INTO @loCols VALUES (25,'IntegerData', 'Integer value dependent on the event class captured in the trace.','int')
 INSERT INTO @loCols VALUES (26,'ServerName', 'Name of the instance of SQL Server, either servername or servername\instancename, being traced.','nvarchar')
 INSERT INTO @loCols VALUES (27,'EventClass', 'Type of event class being recorded.','int')
 INSERT INTO @loCols VALUES (28,'ObjectType', 'Type of object, such as: table, function, or stored procedure.','int')
 INSERT INTO @loCols VALUES (29,'NestLevel', 'The nesting level at which this stored procedure is executing. See @@NESTLEVEL (Transact-SQL).','int')
 INSERT INTO @loCols VALUES (30,'State', 'Server state, in case of an error.','int')
 INSERT INTO @loCols VALUES (31,'Error', 'Error number.','int')
 INSERT INTO @loCols VALUES (32,'Mode', 'Lock mode of the lock acquired. This column is not populated by the Lock:Released event.','int')
 INSERT INTO @loCols VALUES (33,'Handle', 'Handle of the object referenced in the event.','int')
 INSERT INTO @loCols VALUES (34,'ObjectName', 'Name of object accessed.','nvarchar')
 INSERT INTO @loCols VALUES (35,'DatabaseName', 'Name of the database specified in the USE database statement.','nvarchar')
 INSERT INTO @loCols VALUES (36,'FileName', 'Logical name of the file name modified.','nvarchar')
 INSERT INTO @loCols VALUES (37,'OwnerName', 'Owner name of the referenced object.','nvarchar')
 INSERT INTO @loCols VALUES (38,'RoleName', 'Name of the database or server-wide role targeted by a statement.','nvarchar')
 INSERT INTO @loCols VALUES (39,'TargetUserName', 'User name of the target of some action.','nvarchar')
 INSERT INTO @loCols VALUES (40,'DBUserName', 'SQL Server database user name of the client.','nvarchar')
 INSERT INTO @loCols VALUES (41,'LoginSid', 'Security identifier (SID) of the logged-in user.','image')
 INSERT INTO @loCols VALUES (42,'TargetLoginName', 'Login name of the target of some action.','nvarchar')
 INSERT INTO @loCols VALUES (43,'TargetLoginSid', 'SID of the login that is the target of some action.','image')
 INSERT INTO @loCols VALUES (44,'ColumnPermissions', 'Column-level permissions status; used by Security Auditing.','int')
 INSERT INTO @loCols VALUES (45,'LinkedServerName', 'Name of the linked server.','nvarchar')
 INSERT INTO @loCols VALUES (46,'ProviderName', 'Name of the OLE DB provider.','nvarchar')
 INSERT INTO @loCols VALUES (47,'MethodName', 'Name of the OLE DB method.','nvarchar')
 INSERT INTO @loCols VALUES (48,'RowCounts', 'Number of rows in the batch.','bigint')
 INSERT INTO @loCols VALUES (49,'RequestID', 'ID of the request containing the statement.','int')
 INSERT INTO @loCols VALUES (50,'XactSequence', 'A token to describe the current transaction.','bigint')
 INSERT INTO @loCols VALUES (51,'EventSequence', 'Sequence number for this event.','bigint')
 INSERT INTO @loCols VALUES (52,'BigintData1', 'bigint  value, which is dependent on the event class captured in the trace.','bigint')
 INSERT INTO @loCols VALUES (53,'BigintData2', 'bigint  value, which is dependent on the event class captured in the trace.','bigint')
 INSERT INTO @loCols VALUES (54,'GUID', 'GUID value, which is dependent on the event class captured in the trace.','uniqueidentifier')
 INSERT INTO @loCols VALUES (55,'IntegerData2', 'Integer value, which is dependent on the event class captured in the trace.','int')
 INSERT INTO @loCols VALUES (56,'ObjectID2', 'ID of the related object or entity, if available.','bigint')
 INSERT INTO @loCols VALUES (57,'Type', 'Integer value, which is dependent on the event class captured in the trace.','int')
 INSERT INTO @loCols VALUES (58,'OwnerID', 'Type of the object that owns the lock. For lock events only.','int')
 INSERT INTO @loCols VALUES (59,'ParentName', 'Name of the schema the object is within.','nvarchar')
 INSERT INTO @loCols VALUES (60,'IsSystem', 'Indicates whether the event occurred on a system process or a user process.'+CHAR(13)+CHAR(10)+'1 = system'+CHAR(13)+CHAR(10)+'0 = user.','int')
 INSERT INTO @loCols VALUES (61,'Offset', 'Starting offset of the statement within the stored procedure or batch.','int')
 INSERT INTO @loCols VALUES (62,'SourceDatabaseID', 'ID of the database in which the source of the object exists.','int')
 INSERT INTO @loCols VALUES (63,'SqlHandle', '64-bit hash based on the text of an ad hoc query or the database and object ID of an SQL object. This value can be passed to sys.dm_exec_sql_text() to retrieve the associated SQL text.','image')
 INSERT INTO @loCols VALUES (64,'SessionLoginName', 'The login name of the user who originated the session. For example, if you connect to SQL Server using Login1 and execute a statement as Login2, SessionLoginName displays Login1, while LoginName displays Login2. This data column displays both SQL Server and Windows logins.','nvarchar')
 INSERT INTO @loCols VALUES (65,'PlanHandle', '','image')
 INSERT INTO @loCols VALUES (66,'GroupID', '','int')

 RETURN
END --CVP_COLSLIST
GO

/*
Value Definition
8259 Check Constraint 
8260 Default (constraint or standalone) 
8262 Foreign-key Constraint 
8272 Stored Procedure 
8274 Rule 
8275 System Table
8276 Trigger on Server
8277 (User-defined) Table 
8278 View 
8280 Extended Stored Procedure 
16724 CLR Trigger 
16964 Database 
16975 Object 
17222 FullText Catalog 
17232 CLR Stored Procedure 
17235 Schema 
17475 Credential 
17491 DDL Event 
17741 Management Event 
17747 Security Event 
17749 User Event 
17985 CLR Aggregate Function
17993 Inline Table-valued SQL Function 
18000 Partition Function 
18002 Replication Filter Procedure 
18004 Table-valued SQL Function 
18259 Server Role 
18263 Microsoft Windows Group 
19265 Asymmetric Key 
19277 Master Key 
19280 Primary Key 
19283 ObfusKey 
19521 Asymmetric Key Login 
19523 Certificate Login 
19538 Role 
19539 SQL Login 
19543 Windows Login 
20034 Remote Service Binding 
20036 Event Notification on Database
20037 Event Notification 
20038 Scalar SQL Function 
20047 Event Notification on Object
20051 Synonym 
20307 Sequence 
20549 End Point 
20801 Adhoc Queries which may be cached
20816 Prepared Queries which may be cached
20819 Service Broker Service Queue 
20821 Unique Constraint 
21057 Application Role 
21059 Certificate 
21075 Server 
21076 Transact-SQL Trigger 
21313 Assembly 
21318 CLR Scalar Function 
21321 Inline scalar SQL Function 
21328 Partition Scheme 
21333 User 
21571 Service Broker Service Contract 
21572 Trigger on Database
21574 CLR Table-valued Function 
21577 Internal Table (For example, XML Node Table, Queue Table.) 
21581 Service Broker Message Type 
21586 Service Broker Route 
21587 Statistics 
21825
21827
21831
21843
21847 User 
22099 Service Broker Service 
22601 Index 
22604 Certificate Login 
22611 XMLSchema
22868 Type
*/


IF NOT OBJECT_ID('dbo._CVP_STARTTRACE') IS NULL DROP PROCEDURE dbo._CVP_STARTTRACE
GO
CREATE PROCEDURE dbo._CVP_STARTTRACE(@litraceid INT)
AS
BEGIN
 -- Sputn trace logu
 EXEC sp_trace_setstatus @litraceid, 1
END --_CVP_STARTTRACE
GO

IF NOT OBJECT_ID('dbo._CVP_PAUSETRACE') IS NULL DROP PROCEDURE dbo._CVP_PAUSETRACE
GO
CREATE PROCEDURE dbo._CVP_PAUSETRACE(@litraceid INT)
AS
BEGIN
 -- Pozastaven trace logu
 EXEC sp_trace_setstatus @litraceid, 0
END --_CVP_PAUSETRACE
GO

IF NOT OBJECT_ID('dbo._CVP_STOPTRACE') IS NULL DROP PROCEDURE dbo._CVP_STOPTRACE
GO
CREATE PROCEDURE dbo._CVP_STOPTRACE(@litraceid INT)
AS
BEGIN
 -- Zastaven trace logu
 EXEC sp_trace_setstatus @litraceid, 0
 -- Ukonen trace logu
 EXEC sp_trace_setstatus @litraceid, 2
END --_CVP_STOPTRACE
GO

IF NOT OBJECT_ID('dbo._CVP_CREATETRACE') IS NULL DROP PROCEDURE dbo._CVP_CREATETRACE
GO
CREATE PROCEDURE dbo._CVP_CREATETRACE(@lcTRCFILE NVARCHAR(256), @lioptions INT=2, @liMAXFILESIZE BIGINT=1262485504, @litraceid INT OUT)
AS
BEGIN
 -- zaloen trace logu
 EXEC sp_trace_create @litraceid OUTPUT, @lioptions, @lcTRCFILE, @liMAXFILESIZE, NULL, NULL
 PRINT @litraceid
END --_CVP_CREATETRACE
GO

IF NOT OBJECT_ID('dbo._CVP_FILTERTRACE') IS NULL DROP PROCEDURE dbo._CVP_FILTERTRACE
GO
CREATE PROCEDURE dbo._CVP_FILTERTRACE(@litraceid INT, @lcLOPER VARCHAR(3), @lcCOL NVARCHAR(128), @lcCOPER VARCHAR(3), @luVAL SQL_VARIANT)
AS
BEGIN
 -- zaloen trace logu
 DECLARE @liLOPER INT, @liCOPER INT, @lcTYPENAME NVARCHAR(128), @liCOLID INT,
         @lbiVAL BIGINT, @ldVAL DATETIME, @lbVAL VARBINARY, @liVAL INT, @lcVAL NVARCHAR, @lgVAL UNIQUEIDENTIFIER

 SELECT @lcTYPENAME=TYPENAME, @liCOLID=COLID FROM dbo._CVP_COLSLIST() WHERE COLNAME=@lcCOL

 SET @liLOPER=CASE WHEN @lcLOPER='AND' OR @lcLOPER IS NULL THEN 0 ELSE 1 END
 SET @liCOPER=CASE @lcCOPER WHEN '=' THEN 0
                            WHEN '<>' THEN 1
                            WHEN '>' THEN 2
                            WHEN '<' THEN 3
                            WHEN '>=' THEN 4
                            WHEN '<=' THEN 5
                            WHEN 'LIKE' THEN 6  
                            WHEN 'NOT LIKE' THEN 7 ELSE 0 END

 IF @lcTYPENAME='bigint' BEGIN
    SET @lbiVAL=CAST( @luVal AS BIGINT)
    EXEC sp_trace_setfilter @litraceid, @liCOLID,  @liLOPER, @liCOPER, @lbiVAL
 END
 IF @lcTYPENAME='datetime' BEGIN
    SET @ldVAL=CAST( @luVal AS DATETIME)
    EXEC sp_trace_setfilter @litraceid, @liCOLID,  @liLOPER, @liCOPER, @ldVAL
 END
 IF @lcTYPENAME='image' BEGIN
    SET @lbVAL=CAST( @luVal AS VARBINARY(512))
    EXEC sp_trace_setfilter @litraceid, @liCOLID,  @liLOPER, @liCOPER, @lbVAL
 END
 IF @lcTYPENAME='int' BEGIN
    SET @liVAL=CAST( @luVal AS INT)
    EXEC sp_trace_setfilter @litraceid, @liCOLID,  @liLOPER, @liCOPER, @liVAL
 END
 IF @lcTYPENAME='ntext' BEGIN
    SET @lcVAL=CAST( @luVal AS NVARCHAR(512))
    EXEC sp_trace_setfilter @litraceid, @liCOLID,  @liLOPER, @liCOPER, @lcVAL
 END
 IF @lcTYPENAME='nvarchar' BEGIN
    SET @lcVAL=CAST( @luVal AS NVARCHAR(512))
    EXEC sp_trace_setfilter @litraceid, @liCOLID,  @liLOPER, @liCOPER, @lcVAL
 END
 IF @lcTYPENAME='uniqueidentifier' BEGIN
    SET @lgVAL=CAST( @luVal AS UNIQUEIDENTIFIER)
    EXEC sp_trace_setfilter @litraceid, @liCOLID,  @liLOPER, @liCOPER, @lgVAL
 END

END --_CVP_FILTERTRACE
GO


IF NOT OBJECT_ID('dbo._CVP_QUICKTRACE') IS NULL DROP PROCEDURE dbo._CVP_QUICKTRACE
GO
CREATE PROCEDURE dbo._CVP_QUICKTRACE(@lcTRCFILE NVARCHAR(256), @litraceid INT OUT, @lcCOL NVARCHAR(128), @lcCOPER VARCHAR(3), @luVAL SQL_VARIANT, @lbDSQL BIT)
AS
BEGIN
 -- zaloen trace logu
 DECLARE @lioptions INT, @liMAXFILESIZE BIGINT
 SET @lioptions =2
 SET @liMAXFILESIZE=1024*1024*1024
 EXEC sp_trace_create @litraceid OUTPUT, @lioptions, @lcTRCFILE, @liMAXFILESIZE, NULL, NULL
 PRINT @litraceid


 --RPC:Starting	
 IF @lbDSQL=1 BEGIN
    EXEC sp_trace_setevent @litraceid, 11 , 12, 1 --SPID
    EXEC sp_trace_setevent @litraceid, 11 , 51, 1 --EventSequence
    EXEC sp_trace_setevent @litraceid, 11 , 3, 1 --DatabaseID
    EXEC sp_trace_setevent @litraceid, 11 , 49, 1 --RequestID
 END
--SP:StmtStarting
 EXEC sp_trace_setevent @litraceid, 44 , 12, 1 --SPID
 EXEC sp_trace_setevent @litraceid, 44 , 51, 1 --EventSequence
 EXEC sp_trace_setevent @litraceid, 44 , 3, 1 --DatabaseID
 EXEC sp_trace_setevent @litraceid, 44 , 22, 1 --ObjectID
 EXEC sp_trace_setevent @litraceid, 44 , 61, 1 --Offset
 EXEC sp_trace_setevent @litraceid, 44 , 5, 1 --LineNumber
 EXEC sp_trace_setevent @litraceid, 44 , 29, 1 --NestLevel
 EXEC sp_trace_setevent @litraceid, 44 , 49, 1 --RequestID
 EXEC sp_trace_setevent @litraceid, 44 , 30, 1 --State
 IF @lbDSQL=1 BEGIN
    EXEC sp_trace_setevent @litraceid, 44 , 28, 1 --ObjectType
    EXEC sp_trace_setevent @litraceid, 44 , 1, 1 --TextData
 END

--SP:StmtCompleted
 EXEC sp_trace_setevent @litraceid, 45 , 12, 1 --SPID
 EXEC sp_trace_setevent @litraceid, 45 , 51, 1 --EventSequence
 EXEC sp_trace_setevent @litraceid, 45 , 3, 1 --DatabaseID
 EXEC sp_trace_setevent @litraceid, 45 , 22, 1 --ObjectID
 EXEC sp_trace_setevent @litraceid, 45 , 61, 1 --Offset
 EXEC sp_trace_setevent @litraceid, 45 , 5, 1 --LineNumber
 EXEC sp_trace_setevent @litraceid, 45 , 29, 1 --NestLevel
 EXEC sp_trace_setevent @litraceid, 45 , 13, 1 --Duration
 EXEC sp_trace_setevent @litraceid, 45 , 49, 1 --RequestID
 EXEC sp_trace_setevent @litraceid, 45 , 30, 1 --State

 EXEC dbo._CVP_FILTERTRACE @litraceid, NULL, @lcCOL, @lcCOPER, @luVAL
 IF @lbDSQL=0 BEGIN
    EXEC dbo._CVP_FILTERTRACE @litraceid, 'AND', 'ObjectType', '<>', 20801
    EXEC dbo._CVP_FILTERTRACE @litraceid, 'AND', 'ObjectType', '<>', 20816
 END

 -- Sputn trace logu
 EXEC sp_trace_setstatus @litraceid, 1
END --_CVP_QUICKTRACE
GO





/*
exec dbo._CVP_CHECKGRANT


EXEC dbo._CVP_QUICKTRACE '\\server\MSPROFILER\TRC\IFPEUC02_007', NULL, 'SPID', '=', 58,0 -- without dynamic SQL
EXEC dbo._CVP_QUICKTRACE '\\server\MSPROFILER\TRC\IFPEUC02_007', NULL, 'SPID', '=', 58,1 -- with dynamic SQL

EXEC dbo._CVP_STOPTRACE 2

EXEC _CVP_IMPORT_TRC '\\server\MSPROFILER\TRC\IFPEUC02_007.trc', '\\server\MSPROFILER\log\', 'IFPEUC02_007',2

*/

